sets       days                  dates
           stocks                stocks
alias      (stocks,s,t);
parameters val(stocks,days)      closing value
           return(stocks,days)   daily returns
	  mean(stocks)          mean of daily return
           dev(stocks,days)      deviations
           covar(s,t)            covariance matrix of returns (upper)
           totmean               total mean return

$call gams qpdata gdx qpdata
$gdxin qpdata
$load days stocks val

variables  z                     objective variable
           x(stocks)             investments in fraction of budget

semicont variables x; x.lo(s) = 0.1; x.up(s)=1;

equations obj                    objective
          budget                 budget fraction adds up to 1
          retcon                 return minimum;

obj..      sum((s,t), x(s)*covar(s,t)*x(t)) =e= z;
budget..   sum(s, x(s))         =e= 1.0;
retcon..   sum(s, mean(s)*x(s)) =g= totmean*1.25;

model qp1 /all/;

Parameter rep Some simple report;
Set       d(days) all but the last day; d(days-1) = yes;

return(s,days-1) = val(s,days)-val(s,days-1);
mean(s)          = sum(d, return(s,d))/card(d);
dev(s,d)         = return(s,d)-mean(s);
covar(s,t)       = sum(d, dev(s,d)*dev(t,d))/(card(d)-1);
totmean          = sum(s, mean(s))/(card(s));

solve qp1 min z using miqcp;

rep(s) = x.l(s); display rep;