$title Standard QP Model with GDX data input (QP1X,SEQ=246)

* From StockMaster at MIT

sets       days               dates
           stocks             stocks
parameters rval(days, stocks) closing value;

$onechoV > collect.gms
$title collect results
sleeps = 0;
repeat
   loop(scenp$handle(scenp),
      if(handlestatus(handle(scenp))=2,
         qp1.handle = handle(scenp);
         execute_loadhandle qp1;
         rep(scenp,"all","covar")  = z.l;
         rep(scenp,"all","return") = sum(s, mean(s)*x.l(s));
         rep(scenp,"all","resusd") = qp1.resusd;
         rep(scenp,"sol",s)        = x.l(s);
         handle(scenp) = 0) ) ;
   if(card(handle) > %1, display 'go to sleep';  execute 'sleep 1');
   sleeps = sleeps + 1;
until (card(handle) <= %1) or (sleeps >= 400);
$offecho

$onecho > gdxxrw.drv
i=qpdata.xls o=qpdata.gdx
dset=days   rng="Closing values!a2" rdim=1 cdim=0
dset=stocks rng="Closing values!b1" rdim=0 cdim=1
par=val     rng="Closing values!a1" cdim=1 rdim=1
$offecho
$call =gdxxrw @gdxxrw.drv
$if errorlevel 1 $abort 'Problems reading Excel data'.
$gdxin qpdata
$load days stocks rval=val

alias(stocks,s,t); alias(days,d);
Set        upper(s,t)
           lower(s,t);

upper(s,t) = ord(s) <= ord(t);
lower(s,t) = not upper(s,t);

parameter val(stocks,days)      closing values
          return(stocks,days)   daily returns
	  mean(stocks)          mean of daily return
          dev(days,stocks)      deviations
          covar(s,t)            covariance matrix of returns (upper)
          totmean               total mean return;
option val<rval;
$exit
variables z          objective variable
          x(stocks)  investments;

semicont variables x; x.lo(s) = 0.1; x.up(s)=1;


equations obj    objective
          budget
          retcon return constraint;


obj..      z =e= sum(upper(s,t), x(s)*covar(s,t)*x(t)) +
               sum(lower(s,t), x(s)*covar(t,s)*x(t));
budget..   sum(s, x(s)) =e= 1.0;
retcon..   sum(s, mean(s)*x(s)) =g= totmean*1.25;

$set nscen 15
$if not set ncpu $set ncpu 1
model qp1 /all/; option optcr=0, limrow=0, limcol=0;
qp1.solprint=0; qp1.solvelink=2; z.l=0; x.l(s)=0; mean(s)=0;

set scen        scenarios / s1*s%nscen% /; alias (scen,scenp);
Parameter
   rep               some report
   sleeps            number of sleeps / 0 /
   ncpu              number of parallel solves / %ncpu% /
   handle(scen)      scenario handle for solution retrieving
   valx(stocks,days) save the closing values; option valx<val; ncpu = ncpu-1;

if (ncpu > 0, qp1.solvelink=3; else qp1.solvelink=2);
handle(scen)=0;
loop(scen,
  if (ncpu > 0 and card(handle) > ncpu,
$   batinclude collect ncpu
  );
  return(s,d-1)     = val(s,d)-val(s,d-1);
  mean(s)           = sum(d, return(s,d))/card(d);
  dev(d,s)          = return(s,d)-mean(s);
  covar(upper(s,t)) = sum(d, dev(d,s)*dev(d,t))/(card(d)-1);
  totmean           = sum(s, mean(s))/(card(s));

  Solve qp1 using miqcp minimizing z;  handle(scen)=qp1.handle;
  if (ncpu=0,
    rep(scen,"all","covar")  = z.l;
    rep(scen,"all","return") = sum(s, mean(s)*x.l(s));
    rep(scen,"all","resusd") = qp1.resusd;
    rep(scen,"sol",s)        = x.l(s));
  val(s,d) = valx(s,d)*uniform(0.95,1.12);
display val;
);
if (ncpu > 0,
$ batinclude collect 0
);

display rep;
set ss(s); ss(s) = sum(scen$rep(scen,"sol",s),1);
rep(scen,"sol",ss)$(rep(scen,"sol",ss)=0) = eps;

execute "rm -rf %gams.scrdir%grid*"
