sets       days                  dates
           stocks                stocks
alias      (stocks,s,t);
parameters val(stocks,days)      closing value
           return(stocks,days)   daily returns
	  mean(stocks)          mean of daily return
           dev(stocks,days)      deviations
           covar(s,t)            covariance matrix of returns (upper)
           totmean               total mean return

$onecho > gdxxrw.drv
i=qpdata.xls o=qpdata.gdx
dset=days   rng="Closing values!b1" cdim=1 rdim=0
dset=stocks rng="Closing values!a2" cdim=0 rdim=1
par=val     rng="Closing values!a1" cdim=1 rdim=1
$offecho

*$call gdxxrw @gdxxrw.drv
*$call gams qpdata gdx qpdata
$gdxin qpdata
$load days stocks val

variables  z                     objective variable
           x(stocks)             investments  in fraction of budget

semicont variables x; x.lo(s) = 0.1; x.up(s)=1;

equations obj                    objective
          budget                 budget fraction adds up to 1
          retcon                 return minimum;

obj..      sum((s,t), x(s)*covar(s,t)*x(t)) =e= z;
budget..   sum(s, x(s))         =e= 1.0;
retcon..   sum(s, mean(s)*x(s)) =g= totmean*1.25;

model qp1 /all/; qp1.solvelink=3;

Parameter rep         Some report
Set       d(days) all but the last day; d(days-1) = yes;
set m Scenarios / m1*m5 /;

Parameter handle(m)   scenario handle for solution retrieving; handle(m)=0;

loop(m,
  return(s,days-1) = val(s,days)-val(s,days-1);
  mean(s)          = sum(d, return(s,d))/card(d);
  dev(s,d)         = return(s,d)-mean(s);
  covar(s,t)       = sum(d, dev(s,d)*dev(t,d))/(card(d)-1);
  totmean          = sum(s, mean(s))/(card(s));

  solve qp1 min z using miqcp; handle(m)=qp1.handle;

  val(s,days) = uniform(0.9,1.1)*val(s,days);
);

* Collect solutions
repeat
   loop(m$handle(m),
      if(handlestatus(handle(m))=2,
         qp1.handle = handle(m);
         execute_loadhandle qp1;
         rep(m,"all","covar")  = z.l;
         rep(m,"all","return") = sum(s, mean(s)*x.l(s));
         rep(m,"all","resusd") = qp1.resusd;
         rep(m,"sol",s)        = x.l(s);
         handle(m) = 0;
      );
   );
   if(card(handle), display 'go to sleep';  execute 'sleep 1');
until (card(handle) = 0);




execute "rm -rf %gams.scrdir%grid*"
parameter xxxr; set ss(s); xxxr(m,"sol",s) = rep(m,"sol",s); rep(m,"sol",s)=no;
ss(s) = sum(m$xxxr(m,"sol",s),1);
rep(m,"sol",ss) = xxxr(m,"sol",ss); rep(m,"sol",ss)$(rep(m,"sol",ss)=0) = eps;